#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""Constants related to the Azure auth method and/or secrets engine."""

VALID_ENVIRONMENTS = [
    'AzurePublicCloud',
    'AzureUSGovernmentCloud',
    'AzureChinaCloud',
    'AzureGermanCloud',
]
